#/usr/bin/env python

import pygame
import math
import random

class MySpr(pygame.sprite.Sprite):
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)
        self.image = None
        self.rect = pygame.Rect(0,0,0,0)
        self.x = 0.0
        self.y = 0.0
        self.collision = pygame.sprite.Sprite()
        self.collision.rect = pygame.Rect(0,0,0,0)
    
    def update_rect(self):
        self.rect[0] = int(self.x)
        self.rect[1] = int(self.y + screen_y)
        
        self.collision.rect[0] = int(self.x)
        self.collision.rect[1] = int(self.y)
    
    def update(self):
        self.update_rect()
        
    def draw(self, surf):
        surf.blit(self.image, self.rect)
        

class Player(MySpr):
    def __init__(self):
        MySpr.__init__(self)
        self.state = 0
        self.x = 320
        self.y = 240
        self.image = img['player']
        self.rect = self.image.get_rect()
        self.collision.rect[2] = self.rect[2]
        self.collision.rect[3] = self.rect[3]
        
    def update(self, *args):
        global state
        
        key = args[0]
        
        self.y += 5.1
        
        if key[pygame.K_LEFT]:
            self.x -= 4
            self.y -= 0.15
            
        if key[pygame.K_RIGHT]:
            self.x += 4
            self.y -= 0.15
            
        if self.y > (-screen_y + 380):
            self.y = (-screen_y + 380)
            
        if self.y < (-screen_y + 80):
            state = 4
        
        if pygame.sprite.spritecollideany(self.collision, tree_collision):
            self.y -= 1.0
            if use_sound:
                snd['hit'].play()
            
        if self.y > 13000:
            state = 3
            
        self.update_rect()

class Avalanche(MySpr):
    def __init__(self):
        MySpr.__init__(self)
        self.image = img['avalanche']
        self.rect = self.image.get_rect()
        self.collision.rect[2] = self.rect[2]
        self.collision.rect[3] = self.rect[3] - 80
        self.x = 0
        self.y = -80
        
    def update(self, *args):
        self.y += 5
        if (screen_y % 60) == 0:
            self.image = pygame.transform.flip(self.image, True, False)
        self.update_rect()
        
class Tree(MySpr):
    def __init__(self):
        MySpr.__init__(self)
        self.image = img['tree']
        self.rect = self.image.get_rect()
        self.collision.rect[2] = self.rect[2]
        self.collision.rect[3] = self.rect[3]
        self.x = random.randint(0, 640 - self.image.get_width())
        self.y = random.randint(500, 12000)
        tree_collision.add(self.collision)

use_sound = True
pygame.init()
try:
    pygame.mixer.init()
except:
    use_sound = False

screen = pygame.display.set_mode((640, 480))

img = {}
img['title'] = pygame.image.load("title.png")
img['win'] = pygame.image.load("win.png")
img['lose'] = pygame.image.load("lose.png")
img['snow'] = pygame.image.load("snow.png")
img['avalanche'] = pygame.image.load("avalanche.png").convert_alpha()
img['tree'] = pygame.image.load("tree.png").convert_alpha()
img['player'] = pygame.image.load("player.png").convert_alpha()

snd = {}
if use_sound:
    snd['hit'] = pygame.mixer.Sound('hit.wav')

def reset():
    global tree_collision, player, avalanche, trees, screen_y
    
    tree_collision = pygame.sprite.Group()

    player = Player()
    avalanche = Avalanche()
    trees = [Tree() for i in xrange(300)]

    screen_y = 0

reset()

clock = pygame.time.Clock()
state = 1
while state != 0:
    key = pygame.key.get_pressed()
    if state == 1:
        screen.blit(img['title'], (0,0))
        
    elif state == 2:
        screen_y -= 5
        screen.blit(img['snow'], (0, screen_y % 480))
        screen.blit(img['snow'], (0, (screen_y % 480) - 480))
        
        avalanche.update()
        
        for t in trees:
            t.update()
            t.draw(screen)
        
        player.update(key)
        player.draw(screen)
        
        avalanche.draw(screen)
        
    elif state == 3:
        screen.blit(img['win'], (0,0))
    
    elif state == 4:
        screen.blit(img['lose'], (0,0))
    
    clock.tick(60)
    pygame.display.flip()
    
    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            state = 0
            
        if event.type == pygame.KEYDOWN:
            if event.key == pygame.K_SPACE:
                if state == 1:
                    state = 2
                
                if (state == 3) or (state == 4):
                    reset()
                    state = 1
                    
            if event.key == pygame.K_ESCAPE:
                state = 0

pygame.quit()
